'use strict';
function ContentManager($title, $content, MPEditor) {
  this.$title = $title;
  this.$content = $content;
  this.titleMax = 36;
  this.titleMin = 5;
  this.textHeight = 29;
  this.fixHeight = 54;
  this.loadingContent = false;

  var contentManager = this;
  this.$title
    .on('input', function(event) {
      contentManager.correctTitle();
    })
    .on('focus', function(event) {
      SHEditor.triggerTextTagChange(
        Object.myAssign(MPEditor.correctedTagStatus, {
          disabled: true,
        })
      );
    })
    .on('blur', function(event) {
      contentManager.correctTitle(contentManager.titleMax);
      SHEditor.triggerTextTagChange(
        Object.myAssign(MPEditor.correctedTagStatus, {
          disabled: false,
        })
      );
    })
    .focus();
}

ContentManager.prototype.setTitle = function(string) {
  this.$title.val(string);
  this.setTitleHeight();
};

ContentManager.prototype.setContent = function(content) {
  var contentManager = this;
  contentManager.loadingContent = true;
  contentManager.$content.html(content).append('<p></br></p>');
  setTimeout(function() {
    contentManager.loadingContent = false;
  }, 500);
};

ContentManager.prototype.setTitleHeight = function() {
  var $title = this.$title;
  var titleElement = $title[0];
  var offsetHeight = titleElement.offsetHeight;
  var scrollHeight = titleElement.scrollHeight;
  if (offsetHeight <= scrollHeight) {
    $title.css('height', 'auto');
    $title.css('height', `${titleElement.scrollHeight}px`);
  }
};

ContentManager.prototype.correctTitle = function(limit) {
  if (limit === undefined) {
    limit = Infinity;
  }
  var title = this.$title.val();
  var string = MPEditor.computeWordCount(title, limit).string;
  var length = MPEditor.computeWordCount(title, limit).length;
  this.setTitle(string);
  SHEditor.updateTextCount(this.getTextCount());
  var exceedLength = length - this.titleMax;
  var $exceeded = $('#exceeded');
  if (exceedLength > 0) {
    $exceeded.css('display', 'flex').text(`已超出${exceedLength}个字`);
  } else if (length < this.titleMin) {
    $exceeded.css('display', 'flex').text(`标题不可少于${this.titleMin}个字`);
  } else {
    $exceeded.css('display', 'none');
  }
};

ContentManager.prototype.getTextCount = function() {
  var title = this.$title.val();
  var content = this.$content.html();
  var realContent = this.removeEmpty();
  return {
    title: MPEditor.computeWordCount(title).length,
    content: this.computeContentLength(content),
    notEmptyFlag: parseInt(`${realContent ? 1 : 0}${title.length ? 1 : 0}`, 2),
  };
};

ContentManager.prototype.removeEmpty = function() {
  return this.$content
    .html()
    .replace(/<br>/g, '')
    .replace(/<span class="ql-cursor">[\n\r\s]*<\/span>/g, '')
    .replace(/<(strong)>[\n\r\s]*<\/\1>/g, '') // 处理inline
    .replace(/<(h1|li|blockquote|p)>[\n\r\s]*<\/\1>/g, '') // 处理block
    .replace(/<(ul|ol)>[\n\r\s]*<\/\1>/g, '') // 处理列表
    .replace(/<div class="plugin-image-uploading"[\S\s]*?<\/div>/g, '')
    .replace(/<div class="plugin-image-fail"[\S\s]*?<\/div>/g, '');
};

ContentManager.prototype.computeContentLength = function() {
  var count = 0;
  var handledHTML = this.$content
    .html()
    .replace(/<div.*?>.*?<\/div>/g, '')
    .replace(/<mp-plugin.*?>.*?<\/mp-plugin>/g, '')
    .replace(/<span.*?>.*?<\/span>/g, '')
    .replace(/<br>/g, '')
    .replace(/<(strong)>(.*?)<\/\1>/g, '$2')
    .replace(/<(h1|li|blockquote)>(.*?)<\/\1>/g, '<p>$2</p>')
    .replace(/<p>\s*<\/p>/g, '');
  handledHTML.replace(/<p>(.*?)<\/p>/g, function(string, key1) {
    count += MPEditor.computeWordCount(key1).length;
  });
  return count;
};

ContentManager.prototype.getUploadContent = function() {
  return this.removeEmpty(this.$content.html())
    .replace(
      /<p class="plugin-video" contenteditable="false"(.*?)>.*?<\/p>/g,
      '<p class="ql-align-center"><embed $1></p>'
    )
    .replace(
      /<div class="plugin-image-success".*?><img.*?src="(.*?)".*?><p.*?class="plugin-image-success-operates"><span class="img-desc".*?>(.*?)<\/span>[\S\s]*?<\/p><\/div>/g,
      '<p class="ql-align-center"><img src="$1" /><span class="img-desc">$2</span></p>'
    )
    .replace(/<span class="img-desc">\s*<\/span>/g, '')
    .replace(/(<mp-plugin[^>]*>)[\s\S]+?(<\/mp-plugin>)/gi, '$1$2')
    .replace(/[\n\r]/g, '');
};
